@@ -1,18 +1,20 @@
-from bs4 import BeautifulSoup
 import logging
 try:
     from Queue import Queue  # PY2
 except ImportError:
     from queue import Queue  # PY3
-import requests
-from requests.exceptions import RequestException
-from six import iteritems
 from threading import Thread
 try:
     from urlparse import urljoin  # PY2
 except ImportError:
     from urllib.parse import urljoin  # PY3
 
+from bs4 import BeautifulSoup
+import requests
+from requests.exceptions import RequestException
+from six import iteritems
+from six.moves import range
+
 from .sites import get_all_sites
 
 ALL_SITES = get_all_sites()  # All the Craiglist sites
@@ -38,9 +40,9 @@ def get_list_filters(url):
     list_filters = {}
     response = requests_get(url)
     soup = BeautifulSoup(response.content, 'html.parser')
-    for list_filter in soup.find_all('div', {'class': 'attr_picker'}):
+    for list_filter in soup.find_all('div', class_='search-attribute'):
         filter_key = list_filter.attrs['data-attr']
-        filter_labels = soup.find_all('label', {'data-attr': filter_key})
+        filter_labels = list_filter.find_all('label')
         options = [opt.text.strip() for opt in filter_labels]
         list_filters[filter_key] = {'url_key': filter_key, 'value': options}
     return list_filters
@@ -63,9 +65,14 @@ class CraigslistBase(object):
         'search_titles': {'url_key': 'srchType', 'value': 'T'},
         'has_image': {'url_key': 'hasPic', 'value': 1},
         'posted_today': {'url_key': 'postedToday', 'value': 1},
+        'search_distance': {'url_key': 'search_distance', 'value': None},
+        'zip_code': {'url_key': 'postal', 'value': None},
     }
     extra_filters = {}
 
+    # Set to True to subclass defines the customize_results() method
+    custom_result_fields = False
+
     sort_by_options = {
         'newest': 'date',
         'price_asc': 'priceasc',
@@ -98,7 +105,12 @@ class CraigslistBase(object):
 
         list_filters = get_list_filters(self.url)
 
-        self.filters = {}
+        # If a search has few results, results for "similar listings" will
+        # showed. The solution is a bit counter-intuitive, but to force this
+        # not to happen, we set searchNearby=True, but not pass any
+        # nearbyArea=X, thus showing no similar listings.
+        self.filters = {'searchNearby': 1}
+
         for key, value in iteritems((filters or {})):
             try:
                 filter = (self.base_filters.get(key) or
@@ -141,7 +153,7 @@ class CraigslistBase(object):
         sublinks = soup.find('ul', {'class': 'sublinks'})
         return sublinks and sublinks.find('a', text=area) is not None
 
-    def get_results(self, limit=None, sort_by=None, geotagged=False):
+    def get_results(self, limit=None, start=0, sort_by=None, geotagged=False):
         """
         Get results from Craigslist based on the specified filters.
 
@@ -158,8 +170,8 @@ class CraigslistBase(object):
                 self.logger.error(msg)
                 raise ValueError(msg)
 
-        start = 0
-        total_so_far = 0
+        total_so_far = start
+        results_yielded = 0
         total = 0
 
         while True:
@@ -175,8 +187,8 @@ class CraigslistBase(object):
                 totalcount = soup.find('span', {'class': 'totalcount'})
                 total = int(totalcount.text) if totalcount else 0
 
-            for row in soup.find_all('p', {'class': 'row'}):
-                if limit is not None and total_so_far >= limit:
+            for row in soup.find_all('p', {'class': 'result-info'}):
+                if limit is not None and results_yielded >= limit:
                     break
                 self.logger.debug('Processing %s of %s results ...',
                                   total_so_far + 1, total)
@@ -192,12 +204,12 @@ class CraigslistBase(object):
                 else:
                     pl = row.find('span', {'class': 'pl'})
                     datetime = pl.text.split(':')[0].strip() if pl else None
-                price = row.find('span', {'class': 'price'})
-                where = row.find('small')
+                price = row.find('span', {'class': 'result-price'})
+                where = row.find('span', {'class': 'result-hood'})
                 if where:
                     where = where.text.strip()[1:-1]  # remove ()
-                p_span = row.find('span', {'class': 'p'})
-                p_text = p_span.text if p_span else ''
+                tags_span = row.find('span', {'class': 'result-tags'})
+                tags = tags_span.text if tags_span else ''
 
                 result = {'id': id,
                           'name': name,
@@ -205,22 +217,31 @@ class CraigslistBase(object):
                           'datetime': datetime,
                           'price': price.text if price else None,
                           'where': where,
-                          'has_image': 'pic' in p_text,
-                          'has_map': 'map' in p_text,
+                          'has_image': 'pic' in tags,
+                          # TODO: Look into this, looks like all show map now
+                          'has_map': 'map' in tags,
                           'geotag': None}
 
+                if self.custom_result_fields:
+                    self.customize_result(result, row)
+
                 if geotagged and result['has_map']:
                     self.geotag_result(result)
 
                 yield result
+                results_yielded += 1
                 total_so_far += 1
 
-            if total_so_far == limit:
+            if results_yielded == limit:
                 break
             if (total_so_far - start) < RESULTS_PER_REQUEST:
                 break
             start = total_so_far
 
+    def customize_result(self, result, html_row):
+        """ Add custom/delete/alter fields to result. """
+        pass  # Override in subclass to add category-specific fields.
+
     def geotag_result(self, result):
         """ Adds (lat, lng) to result. """
 
@@ -260,7 +281,7 @@ class CraigslistBase(object):
                 queue.task_done()
 
         threads = []
-        for _ in xrange(workers):
+        for _ in range(workers):
             thread = Thread(target=geotagger)
             thread.start()
             threads.append(thread)
@@ -270,7 +291,7 @@ class CraigslistBase(object):
         return results
 
     @classmethod
-    def show_filters(cls):
+    def show_filters(cls, category=None):
         print('Base filters:')
         for key, options in iteritems(cls.base_filters):
             value_as_str = '...' if options['value'] is None else 'True/False'
@@ -279,8 +300,10 @@ class CraigslistBase(object):
         for key, options in iteritems(cls.extra_filters):
             value_as_str = '...' if options['value'] is None else 'True/False'
             print('* %s = %s' % (key, value_as_str))
-        url = cls.url_templates['no_area'] % {'site': cls.default_site,
-                                              'category': cls.default_category}
+        url = cls.url_templates['no_area'] % {
+            'site': cls.default_site,
+            'category': category or cls.default_category,
+        }
         list_filters = get_list_filters(url)
         for key, options in iteritems(list_filters):
             value_as_str = ', '.join([repr(opt) for opt in options['value']])
@@ -326,14 +349,14 @@ class CraigslistForSale(CraigslistBase):
     default_category = 'sss'
 
     extra_filters = {
-        'min_price': {'url_key': 'minAsk', 'value': None},
-        'max_price': {'url_key': 'maxAsk', 'value': None},
-        'make': {'url_key': 'autoMakeModel', 'value': None},
-        'model': {'url_key': 'autoMakeModel', 'value': None},
-        'min_year': {'url_key': 'autoMinYear', 'value': None},
-        'max_year': {'url_key': 'autoMaxYear', 'value': None},
-        'min_miles': {'url_key': 'autoMilesMin', 'value': None},
-        'max_miles': {'url_key': 'autoMilesMax', 'value': None},
+        'min_price': {'url_key': 'min_price', 'value': None},
+        'max_price': {'url_key': 'max_price', 'value': None},
+        'make': {'url_key': 'auto_make_model', 'value': None},
+        'model': {'url_key': 'auto_make_model', 'value': None},
+        'min_year': {'url_key': 'min_auto_year', 'value': None},
+        'max_year': {'url_key': 'max_auto_year', 'value': None},
+        'min_miles': {'url_key': 'min_auto_miles', 'value': None},
+        'max_miles': {'url_key': 'max_auto_miles', 'value': None},
     }
 
 
@@ -359,24 +382,39 @@ class CraigslistHousing(CraigslistBase):
     """ Craigslist housing wrapper. """
 
     default_category = 'hhh'
+    custom_result_fields = True
 
     extra_filters = {
         'private_room': {'url_key': 'private_room', 'value': 1},
         'private_bath': {'url_key': 'private_bath', 'value': 1},
         'cats_ok': {'url_key': 'pets_cat', 'value': 1},
         'dogs_ok': {'url_key': 'pets_dog', 'value': 1},
-        'min_price': {'url_key': 'minAsk', 'value': None},
-        'max_price': {'url_key': 'maxAsk', 'value': None},
+        'min_price': {'url_key': 'min_price', 'value': None},
+        'max_price': {'url_key': 'max_price', 'value': None},
         'min_ft2': {'url_key': 'minSqft', 'value': None},
         'max_ft2': {'url_key': 'maxSqft', 'value': None},
-        'search_distance': {'url_key': 'search_distance', 'value': None},
-        'zip_code': {'url_key': 'postal', 'value': None},
-        'bedrooms': {'url_key': 'bedrooms', 'value': None},
-        'bathrooms': {'url_key': 'bathrooms', 'value': None},
-        'laundry_in_unit': {'url_key': 'laundry', 'value': 1},
+        'min_bedrooms': {'url_key': 'min_bedrooms', 'value': None},
+        'max_bedrooms': {'url_key': 'max_bedrooms', 'value': None},
+        'min_bathrooms': {'url_key': 'min_bathrooms', 'value': None},
+        'max_bathrooms': {'url_key': 'max_bathrooms', 'value': None},
         'no_smoking': {'url_key': 'no_smoking', 'value': 1},
+        'is_furnished': {'url_key': 'is_furnished', 'value': 1},
+        'wheelchair_acccess': {'url_key': 'wheelchaccess', 'value': 1},
     }
 
+    def customize_result(self, result, html_row):
+        housing_info = html_row.find('span', {'class': 'housing'})
+        # Default values
+        result.update({'bedrooms': None, 'area': None})
+        if housing_info:
+            for elem in housing_info.text.split('-'):
+                elem = elem.strip()
+                if elem.endswith('br'):
+                    # Don't convert to int, too risky
+                    result['bedrooms'] = elem[:-2]
+                if elem.endswith('2'):
+                    result['area'] = elem
+
 
 class CraigslistJobs(CraigslistBase):
     """ Craigslist jobs wrapper. """
@@ -384,10 +422,8 @@ class CraigslistJobs(CraigslistBase):
     default_category = 'jjj'
 
     extra_filters = {
-        'is_contract': {'url_key': 'is_contract', 'value': 1},
         'is_internship': {'url_key': 'is_internship', 'value': 1},
         'is_nonprofit': {'url_key': 'is_nonprofit', 'value': 1},
-        'is_parttime': {'url_key': 'is_parttime', 'value': 1},
         'is_telecommuting': {'url_key': 'is_telecommuting', 'value': 1},
     }
 
@@ -398,8 +434,8 @@ class CraigslistPersonals(CraigslistBase):
     default_category = 'ppp'
 
     extra_filters = {
-        'min_age': {'url_key': 'minAsk', 'value': None},
-        'max_age': {'url_key': 'maxAsk', 'value': None},
+        'min_age': {'url_key': 'min_pers_age', 'value': None},
+        'max_age': {'url_key': 'max_pers_age', 'value': None},
     }
 
 
